@@ -44,11 +44,12 @@ For to temporarily stop the sideloading server simple call the :meth:`~Sideloadi
 exit additionally call the method :meth:`~SideloadingServerApp.shutdown`.
 
 .. hint::
-    The ae namespace portion :mod:`ae.kivy_sideloading` is providing a package for an easy integration of this
+    The ae namespace portion :mod:`~ae.kivy_sideloading` is providing a package for an easy integration of this
     sideloading server into your kivy app.
 
     An example usage of this sideloading server and the :mod:`ae.kivy_sideloading` package is integrated into the demo
-    apps `Lisz <https://gitlab.com/ae-group/kivy_lisz>`_ and `ComPartY <https://gitlab.com/ae-group/comparty>`_.
+    apps `GlslTester <https://github.com/AndiEcker/glsl_tester>`_, `Lisz <https://gitlab.com/ae-group/kivy_lisz>`_ and
+    `ComPartY <https://gitlab.com/ae-group/comparty>`_.
 
 """
 import datetime
@@ -69,15 +70,16 @@ from ae.paths import norm_path                          # type: ignore
 from ae.console import ConsoleApp                       # type: ignore
 
 
-__version__ = '0.1.5'
+__version__ = '0.1.8'
 
 
 DEFAULT_FILE_MASK = "{downloads}/{main_app_name}*.apk"     #: default glob file mask for to sideloading file
+FILE_COUNT_MISMATCH = " files found matching "  #: err msg part of :meth:`SideloadingServerApp.start_server`
 
 SERVER_BIND = ""                    #: setting BIND to '' or None for to allow connections for all local devices
 SERVER_PORT = 36996                 #: http server listening port (a port number under 1024 requires root privileges)
 
-SOCKET_BUF_LEN = 4096               #: buf length for socket sends in :meth:`~SideloadingServerApp.response_to_request`
+SOCKET_BUF_LEN = 16 * 1024          #: buf length for socket sends in :meth:`~SideloadingServerApp.response_to_request`
 
 SHUTDOWN_TIMEOUT = 3.9              #: timeout (in seconds) to shutdown/stop the console app and http sideloading server
 
@@ -99,6 +101,7 @@ def server_factory(task_id_func: Optional[Callable[[str, str, str], str]] = None
     server_app.add_option('bind', "server bind address", SERVER_BIND, 'b')
     server_app.add_option('port', "server listening port", SERVER_PORT, 'p')
     server_app.add_option('file_mask', "glob file mask for the sideloading file", DEFAULT_FILE_MASK, 'm')
+    server_app.add_option('buf_len', "socket buffer length", SOCKET_BUF_LEN, 'l')
 
     if task_id_func:
         # noinspection PyTypeHints
@@ -187,7 +190,7 @@ class SideloadingServerApp(ConsoleApp):
         if callable(out_method):
             out_method(("" if self.active_log_stream else f"{threading.current_thread().name: <15}") + f"{message}")
 
-        if getattr(self, log_level, True):
+        if getattr(self, log_level, True) and hasattr(self, 'load_requests'):
             log_time = datetime.datetime.now()
 
             requests_lock.acquire()
@@ -237,8 +240,8 @@ class SideloadingServerApp(ConsoleApp):
                 handler.end_headers()
 
                 progress_kwargs = dict(client_ip=client_ip, handler=handler)
-                copy_bytes(file_handle, handler.wfile, total_bytes=file_size, buf_size=SOCKET_BUF_LEN, errors=errors,
-                           progress_func=self.progress_callback, **progress_kwargs)
+                copy_bytes(file_handle, handler.wfile, total_bytes=file_size, buf_size=self.get_opt('buf_len'),
+                           errors=errors, progress_func=self.progress_callback, **progress_kwargs)
                 if errors:
                     self.log('print', f"{pre} copy_bytes errors: {errors}")
                 else:
@@ -257,7 +260,7 @@ class SideloadingServerApp(ConsoleApp):
         :return:                server url string.
         """
         url = "http://" + os_local_ip()
-        port = self.get_option('port')
+        port = self.get_opt('port')
         if port != 80:
             url += ":" + str(port)
         return url
@@ -272,7 +275,7 @@ class SideloadingServerApp(ConsoleApp):
         super().shutdown(exit_code=exit_code, timeout=timeout)
 
     def start_server(self, file_mask: str = "", threaded: bool = False, progress: Callable = update_handler_progress
-                     ) -> bool:
+                     ) -> str:
         """ start http file sideloading server to run until :meth:`~.stop_server` get called.
 
         :param file_mask:       optional glob.glob file mask to specify the sideloading file. If not passed then
@@ -286,7 +289,11 @@ class SideloadingServerApp(ConsoleApp):
                                 then the default callback method :func:`update_handler_progress`
                                 will be called for to update the progress attributes of the request handler, which can
                                 be polled via the :meth:`~SideloadingServerApp.client_progress` method.
-        :return:                True if server instance/thread got started else False.
+        :return:                empty string/"" if server instance/thread got started else the error message string.
+
+        .. note::
+            if :paramref:`~.start_server.threaded` is `True` then the :paramref:`~.start_server.progress` callback get
+            executed in its own thread.
         """
         pre = "SideloadingServerApp.start_server()"
         if not file_mask:
@@ -298,7 +305,7 @@ class SideloadingServerApp(ConsoleApp):
         self.load_requests = list()
 
         if requests_lock.locked():  # pragma: no cover
-            requests_lock.release()             # release before self.log() call (new aquiring)
+            requests_lock.release()             # release before self.log() call (new acquiring)
             self.log('print', f"{pre}: released requests lock")
         if clients_lock.locked():   # pragma: no cover
             self.log('print', f"{pre}: releasing clients lock")
@@ -306,15 +313,17 @@ class SideloadingServerApp(ConsoleApp):
 
         self.log('debug', f"{pre}: {'threaded' if threaded else ''} with file mask='{file_mask}' and cb={progress}")
 
+        err_msg = ""
         try:
             files = glob.glob(norm_path(file_mask))
             file_count = len(files)
-            assert file_count == 1, f"{pre}: {str(file_count) if file_count else 'no'} files found matching {file_mask}"
+            if file_count != 1:
+                return f"{pre}: {str(file_count) if file_count else 'no'}{FILE_COUNT_MISMATCH}{file_mask}"
             self.file_path = files[0]
 
-            server_address = (self.get_option('bind'), self.get_option('port'))
+            server_address = (self.get_opt('bind'), self.get_opt('port'))
+            HTTPServer.allow_reuse_address = True
             SimpleHTTPRequestHandler.protocol_version = "HTTP/1.0"
-            assert HTTPServer.allow_reuse_address
             self.server_instance = HTTPServer(server_address, SimpleHTTPRequestHandler)
             self.server_instance.timeout = SHUTDOWN_TIMEOUT
 
@@ -333,16 +342,17 @@ class SideloadingServerApp(ConsoleApp):
                 self.server_instance.serve_forever()
 
         except (IOError, OSError, Exception) as ex:
-            self.log('print', f"{pre}: exception {ex}")
+            err_msg = f"{pre}: exception {ex}"
+            self.log('print', err_msg)
             self.server_instance = self.server_thread = None
 
-        return bool(self.server_instance and self.server_thread)
+        return err_msg
 
     def stop_server(self):
         """ stop/pause sideloading server - callable also if not running to reset/prepare this app instance. """
         pre = "SideloadingServerApp.stop_server()"
         if requests_lock.locked():  # pragma: no cover
-            requests_lock.release()             # release before self.log() call (new aquiring)
+            requests_lock.release()             # release before self.log() call (new acquiring)
             self.log('print', f"{pre}: released requests lock")
         if clients_lock.locked():   # pragma: no cover
             self.log('print', f"{pre}: releasing clients lock")
